@@ -30,11 +30,12 @@
 #include "gdm-display-factory.h"
 #include "gdm-display-store.h"
 
-typedef struct _GdmDisplayFactoryPrivate
+#define GDM_DISPLAY_FACTORY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_DISPLAY_FACTORY, GdmDisplayFactoryPrivate))
+
+struct GdmDisplayFactoryPrivate
 {
         GdmDisplayStore *display_store;
-        guint            purge_displays_id;
-} GdmDisplayFactoryPrivate;
+};
 
 enum {
         PROP_0,
@@ -45,7 +46,7 @@ static void     gdm_display_factory_class_init  (GdmDisplayFactoryClass *klass);
 static void     gdm_display_factory_init        (GdmDisplayFactory      *factory);
 static void     gdm_display_factory_finalize    (GObject                *object);
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GdmDisplayFactory, gdm_display_factory, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE (GdmDisplayFactory, gdm_display_factory, G_TYPE_OBJECT)
 
 GQuark
 gdm_display_factory_error_quark (void)
@@ -58,56 +59,12 @@ gdm_display_factory_error_quark (void)
         return ret;
 }
 
-static gboolean
-purge_display (char       *id,
-               GdmDisplay *display,
-               gpointer    user_data)
-{
-        int status;
-
-        status = gdm_display_get_status (display);
-
-        switch (status) {
-        case GDM_DISPLAY_FINISHED:
-        case GDM_DISPLAY_FAILED:
-                return TRUE;
-        default:
-                return FALSE;
-        }
-}
-
-static void
-purge_displays (GdmDisplayFactory *factory)
-{
-        GdmDisplayFactoryPrivate *priv;
-
-        priv = gdm_display_factory_get_instance_private (factory);
-        priv->purge_displays_id = 0;
-        gdm_display_store_foreach_remove (priv->display_store,
-                                          (GdmDisplayStoreFunc)purge_display,
-                                          NULL);
-}
-
-void
-gdm_display_factory_queue_purge_displays (GdmDisplayFactory *factory)
-{
-        GdmDisplayFactoryPrivate *priv;
-
-        priv = gdm_display_factory_get_instance_private (factory);
-        if (priv->purge_displays_id == 0) {
-                priv->purge_displays_id = g_idle_add ((GSourceFunc) purge_displays, factory);
-        }
-}
-
 GdmDisplayStore *
 gdm_display_factory_get_display_store (GdmDisplayFactory *factory)
 {
-        GdmDisplayFactoryPrivate *priv;
-
         g_return_val_if_fail (GDM_IS_DISPLAY_FACTORY (factory), NULL);
 
-        priv = gdm_display_factory_get_instance_private (factory);
-        return priv->display_store;
+        return factory->priv->display_store;
 }
 
 gboolean
@@ -142,13 +99,13 @@ static void
 gdm_display_factory_set_display_store (GdmDisplayFactory *factory,
                                        GdmDisplayStore   *display_store)
 {
-        GdmDisplayFactoryPrivate *priv;
-
-        priv = gdm_display_factory_get_instance_private (factory);
-        g_clear_object (&priv->display_store);
+        if (factory->priv->display_store != NULL) {
+                g_object_unref (factory->priv->display_store);
+                factory->priv->display_store = NULL;
+        }
 
         if (display_store != NULL) {
-                priv->display_store = g_object_ref (display_store);
+                factory->priv->display_store = g_object_ref (display_store);
         }
 }
 
@@ -179,14 +136,12 @@ gdm_display_factory_get_property (GObject    *object,
                                   GParamSpec *pspec)
 {
         GdmDisplayFactory *self;
-        GdmDisplayFactoryPrivate *priv;
 
         self = GDM_DISPLAY_FACTORY (object);
-        priv = gdm_display_factory_get_instance_private (self);
 
         switch (prop_id) {
         case PROP_DISPLAY_STORE:
-                g_value_set_object (value, priv->display_store);
+                g_value_set_object (value, self->priv->display_store);
                 break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -209,32 +164,28 @@ gdm_display_factory_class_init (GdmDisplayFactoryClass *klass)
                                                               "display store",
                                                               "display store",
                                                               GDM_TYPE_DISPLAY_STORE,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+        g_type_class_add_private (klass, sizeof (GdmDisplayFactoryPrivate));
 }
 
 static void
 gdm_display_factory_init (GdmDisplayFactory *factory)
 {
+        factory->priv = GDM_DISPLAY_FACTORY_GET_PRIVATE (factory);
 }
 
 static void
 gdm_display_factory_finalize (GObject *object)
 {
         GdmDisplayFactory *factory;
-        GdmDisplayFactoryPrivate *priv;
 
         g_return_if_fail (object != NULL);
         g_return_if_fail (GDM_IS_DISPLAY_FACTORY (object));
 
         factory = GDM_DISPLAY_FACTORY (object);
-        priv = gdm_display_factory_get_instance_private (factory);
 
-        g_return_if_fail (priv != NULL);
-
-        if (priv->purge_displays_id != 0) {
-                g_source_remove (priv->purge_displays_id);
-                priv->purge_displays_id = 0;
-        }
+        g_return_if_fail (factory->priv != NULL);
 
         G_OBJECT_CLASS (gdm_display_factory_parent_class)->finalize (object);
 }
