@@ -430,7 +430,14 @@ activate_session_id_for_ck (GdmManager *manager,
                 g_debug ("GdmManager: ConsoleKit %s raised:\n %s\n\n",
                          g_dbus_error_get_remote_error (error), error->message);
                 g_error_free (error);
-                return FALSE;
+
+                /* It is very likely that the "error" just reported is
+                 * that the session is already active.  Unfortunately,
+                 * ConsoleKit doesn't use proper error codes and it
+                 * translates the error message, so we have no real way
+                 * to detect this case...
+                 */
+                return TRUE;
         }
 
         g_variant_unref (reply);
@@ -807,7 +814,7 @@ get_display_and_details_for_bus_sender (GdmManager       *self,
         }
 
         if (out_uid != NULL) {
-                *out_uid = session_uid;
+                *out_uid = caller_uid;
         }
 
         if (caller_uid != session_uid) {
@@ -1000,6 +1007,47 @@ on_reauthentication_client_disconnected (GdmSession              *session,
         close_transient_session (self, session);
 }
 
+static void
+on_reauthentication_client_rejected (GdmSession              *session,
+                                     GCredentials            *credentials,
+                                     GPid                     pid_of_client,
+                                     GdmManager              *self)
+{
+        GPid pid;
+
+        g_debug ("GdmManger: client with pid %ld rejected from reauthentication server", (long) pid_of_client);
+
+        if (gdm_session_client_is_connected (session)) {
+                /* we already have a client connected, ignore this rejected one */
+                return;
+        }
+
+        pid = (GPid) GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (session), "caller-pid"));
+
+        if (pid != pid_of_client) {
+                const char *session_id;
+                char *client_session_id;
+
+                /* rejected client isn't the process that started the
+                 * transient reauthentication session. If it's not even from the
+                 * same audit session, ignore it since it doesn't "own" the
+                 * reauthentication session
+                 */
+                client_session_id = get_session_id_for_pid (self->priv->connection,
+                                                            pid_of_client,
+                                                            NULL);
+                session_id = g_object_get_data (G_OBJECT (session), "caller-session-id");
+
+                if (g_strcmp0 (session_id, client_session_id) != 0) {
+                        return;
+                }
+        }
+
+        /* client was rejected, so clean up its session object
+         */
+        close_transient_session (self, session);
+}
+
 static void
 on_reauthentication_cancelled (GdmSession *session,
                                GdmManager *self)
@@ -1091,6 +1139,10 @@ open_temporary_reauthentication_channel (GdmManager            *self,
                           "client-disconnected",
                           G_CALLBACK (on_reauthentication_client_disconnected),
                           self);
+        g_signal_connect (session,
+                          "client-rejected",
+                          G_CALLBACK (on_reauthentication_client_rejected),
+                          self);
         g_signal_connect (session,
                           "cancelled",
                           G_CALLBACK (on_reauthentication_cancelled),
@@ -1144,7 +1196,7 @@ gdm_manager_handle_open_reauthentication_channel (GdmDBusManager        *manager
                 return TRUE;
         }
 
-        if (seat_id == NULL || session_id == NULL || pid == 0 || uid == (uid_t) -1) {
+        if (session_id == NULL || pid == 0 || uid == (uid_t) -1) {
                 g_dbus_method_invocation_return_error_literal (invocation,
                                                                G_DBUS_ERROR,
                                                                G_DBUS_ERROR_ACCESS_DENIED,
